package avatartranslator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AvatarDataType {
    public final static int INTEGER = 0;
    public final static int BOOLEAN = 1;
    public final static int DATATYPE = 2;
    public static Map<String,AvatarDataType> dataTypeMap = new HashMap<String,AvatarDataType>();
    private String name;
    private List<String> fieldStrings;
    private List<Integer> fieldStringTypes;
    private List<String> fieldNames;
    private List<Integer> fieldTypes;
    private List<String> dataTypeNames;
    private int fullSize;

    public AvatarDataType(String _name) {
        name = _name;
        fieldStrings = new ArrayList<String>();
        fieldStringTypes = new ArrayList<Integer>();
        fieldNames = new ArrayList<String>();
        fieldTypes = new ArrayList<Integer>();
        dataTypeNames = new ArrayList<String>();
        fullSize = -1;
    }
    public String getName() { return name; }
    public int getFullSize() { return fullSize; }
    public int getNbFields() { return fieldNames.size(); }
    public String getFieldName(int _i) { return fieldNames.get(_i); }
    public String getFieldString(int _i) { return fieldStrings.get(_i); }
    public int getFieldStringType(int _i) { return fieldStringTypes.get(_i); }
    public int getFieldType(int _i) { return fieldTypes.get(_i); }
    public String getDataTypeName(int _i) { return dataTypeNames.get(_i); }
    public void addField(String _fieldName, int _fieldType, String _dataTypeName) {
        if (fullSize != -1) return; // DataType has already been finalized
        if (_fieldType == AvatarDataType.INTEGER || _fieldType == AvatarDataType.BOOLEAN)
            dataTypeNames.add(null);
        else if (_fieldType != DATATYPE)
            return;
        else
            dataTypeNames.add(_dataTypeName);
        fieldTypes.add(_fieldType);
        fieldNames.add(_fieldName);
    }
    public static void finalize(AvatarDataType adt) {
        if (adt.fullSize != -1) return;
        int size = 0;
        int length = adt.getNbFields();
        for (int i = 0; i < length; i++) {
            if (adt.getFieldType(i) == AvatarDataType.INTEGER) {
                adt.fieldStrings.add(adt.fieldNames.get(i));
                adt.fieldStringTypes.add(AvatarDataType.INTEGER);
                size += 1;
                continue;
            }
            if (adt.getFieldType(i) == AvatarDataType.BOOLEAN) {
                adt.fieldStrings.add(adt.fieldNames.get(i));
                adt.fieldStringTypes.add(AvatarDataType.BOOLEAN);
                size += 1;
                continue;
            }
            AvatarDataType fieldDataType = dataTypeMap.get(adt.getDataTypeName(i));
            if (fieldDataType == null)  return; // ERROR CASE
            finalize(fieldDataType); // loops if recursion => recursive types are forbidden
            int fieldSize = fieldDataType.getFullSize();  // ERROR CASE
            if (fieldSize == -1)  return;
            size += fieldSize;
            for(int j=0; j<fieldSize; j++) {
                adt.fieldStrings.add(adt.fieldNames.get(i) + "__" + fieldDataType.getFieldString(i));
                adt.fieldStringTypes.add(fieldDataType.getFieldStringType(i));
            }
        }
        adt.fullSize = size;
    }
}
